# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

from aostools import *
import os
import shutil

class Pack(Command):
    common = True
    helpSummary = "Compress component to dest directory"
    helpUsage = """
%prog <component> [<component> <component> ...] <dest path>
"""
    helpDescription = """
Compress component to dest directory
"""

    def Execute(self, opt, args):
        if len(args) < 2:
            self.Usage()

        yoc = YoC()
        componentList = args[:-1]
        validComponentList = []
        destPath = args[-1]
        destPath = os.path.abspath(destPath)

        for component in yoc.components:
            if component.name in componentList:
                component.load_package()
                self.component_export(component, destPath)
                validComponentList.append(component.name)
        invalidComponentList = set(componentList).difference(set(validComponentList))
        if invalidComponentList:
            put_string("These components [%s] are not existed!" % (", ").join(invalidComponentList))
        return 0


    def component_export(self, component, path):
        p = get_cmpt_path_by_type(component.type)
        if p:
            groupdir = os.path.join(path, get_cmpt_top_path_by_type(component.type))
            compdir = os.path.join(path, p, component.name)
            gitdir = os.path.join(compdir, '.git')
            try:
                # copy component files
                if os.path.exists(groupdir):
                    rmtree_enhanced(groupdir)
                shutil.copytree(component.path, compdir)
                # remove .git
                if os.path.exists(gitdir):
                    rmtree_enhanced(gitdir)
                # compress
                zipName = os.path.join(path, "%s-%s.zip" % (component.name, component.version))
                put_string("Pack `%s` to %s." % (component.name, zipName))
                if os.path.exists(zipName):
                    os.remove(zipName)
                zip_path(groupdir, zipName)
                rmtree_enhanced(groupdir)
            except Exception as ex:
                put_string(str(ex))
                pass



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

import os
import stat
import tarfile
import subprocess
import platform
import codecs

from .tools import *

qemu_csky_32 = ""
qemu_csky_64 = "http://yoctools.oss-cn-beijing.aliyuncs.com/csky-qemu-x86_64-Ubuntu-16.04-20200901-1526.tar.gz"

qemu_rv_32 = ""
qemu_rv_64 = "http://yoctools.oss-cn-beijing.aliyuncs.com/csky-qemu-x86_64-Ubuntu-16.04-20200901-1526.tar.gz"


all_toolchain_url = {
    'csky-qemu': [qemu_csky_32, qemu_csky_64],
    'riscv-qemu': [qemu_rv_32, qemu_rv_64]
}


class QemuYoC:
    def __init__(self):
        if os.getuid() != 0:
            self.basepath = home_path('.thead')
        else:
            self.basepath = '/usr/local/thead/'

    def download(self):
        qemu_path = os.path.join(self.basepath, self.qemu_arch)

        if os.path.exists(qemu_path):
            self.update_env()
            return

        architecture = platform.architecture()
        if architecture[0] == '64bit':
            qemu_url = all_toolchain_url[self.qemu_arch][1]
        else:
            qemu_url = all_toolchain_url[self.qemu_arch][0]

        tar_path = '/tmp/' + os.path.basename(qemu_url)
        if not qemu_url:
            put_string("Url is empty!")
            return
        put_string("Start to download qemu: %s" % self.qemu_arch)
        wget(qemu_url, tar_path)
        put_string("")
        put_string("Start install, wait half a minute please.")
        if tar_path.endswith('.bz2'):
            with tarfile.open(tar_path, 'r:bz2') as tar:
                tar.extractall(qemu_path)
        elif tar_path.endswith('.gz'):
            with tarfile.open(tar_path, 'r:gz') as tar:
                tar.extractall(qemu_path)
        else:
            put_string("%s extra not support." % tar_path)
            return

        os.remove(tar_path)
        if os.getuid() == 0:
            self.link_bin(qemu_path)
        else:
            self.update_env()
        put_string("Congratulations!")

    def link_bin(self, qemu_path):
        qemu_bin = os.path.join(qemu_path, 'bin')
        files = os.listdir(qemu_bin)

        for fil in files:
            p = os.path.join(qemu_bin, fil)
            if os.path.isfile(p):
                if os.stat(p).st_mode & (stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH) != 0:
                    try:
                        os.symlink(p, os.path.join(
                            '/usr/bin', os.path.basename(p)))
                    except FileExistsError:
                        pass
                    except PermissionError:
                        put_string("Please use: sudo", ' '.join(sys.argv))
                        exit(-1)
                    except Exception as e:
                        pass

    def update_env(self):
        qemu_path = '$HOME/.thead/%s/bin' % self.qemu_arch
        shell = os.getenv('SHELL')
        if not shell:
            shell = 'bash'
        else:
            shell = os.path.basename(shell)

        if shell == 'zsh':
            rc = home_path('.zshrc')

        elif shell == 'bash':
            rc = home_path('.bashrc')

        with codecs.open(rc, 'r', 'UTF-8') as f:
            contents = f.readlines()

        export_path = ''
        for i in range(len(contents)):
            c = contents[i]
            idx = c.find(' PATH')
            if idx > 0:
                idx = c.find('=')
                if idx >= 0:
                    export_path = c[idx + 1:]

                    if export_path.find(qemu_path) < 0:
                        export_path = 'export PATH=' + qemu_path + ':' + export_path
                        contents[i] = export_path

        if not export_path:
            contents.insert(0, 'export PATH=' + qemu_path + ':$PATH\n\n')

        with codecs.open(rc, 'w', 'UTF-8') as f:
            contents = f.writelines(contents)

    def check_qemu(self, arch, verbose=0):
        if arch == 'csky':
            self.qemu_arch = 'csky-qemu'
        elif arch == 'riscv':
            self.qemu_arch = 'riscv-qemu'
        elif arch == 'arm':
            self.qemu_arch = 'arm-qemu'
            exe_file = 'qemu-system-arm'
        else:
            put_string("Not support %s qemu yet." % arch)
            exit(1)
        bin_file = self.check_program()

        if bin_file == '':
            if arch == 'arm':
                put_string('Please use shell cmd to install qemu as follow:\nsudo apt install qemu-system-arm')
                exit(1)            
            self.download()
            bin_file = self.check_program()
        else:
            if verbose == 1:
                put_string('warn: the toolchains was installed already, path = %s.' % bin_file)
        return bin_file

    def which(self, cmd):
        gcc = subprocess.Popen('which ' + cmd, shell=True,
                               stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        lines = gcc.stdout.readlines()
        for text in lines:
            text = text.decode().strip()
            info = 'which: no ' + os.path.basename(cmd) + ' in'
            if not text.find(info) >= 0:
                return text
        return ''

    def check_program(self):
        if self.qemu_arch == 'csky-qemu':
            exe_file = 'qemu-system-cskyv2'
        elif self.qemu_arch == 'riscv-qemu':
            exe_file = 'qemu-system-riscv32'
        elif self.qemu_arch == 'arm-qemu':
            exe_file = 'qemu-system-arm'

        path = self.which(exe_file)
        if not path:
            # put_string("Can't find %s" % exe_file)
            path = home_path('.thead/' + self.qemu_arch + '/bin/' + exe_file)
            path = self.which(path)
        return path



import codecs
import sys
import ssl
try:
    _create_unverified_https_context = ssl._create_unverified_context
except AttributeError:
    # Legacy Python that doesn't verify HTTPS certificates by default
    pass
else:
    # Handle target environment that doesn't support HTTPS verification
    ssl._create_default_https_context = _create_unverified_https_context
# ssl._create_default_https_context = ssl._create_unverified_context

from .builder import *
from .component import *
from .log import logger
from .make import *
from .occ import *
from .package import *
from .manifest import *
from .gitproject import *
from .toolchain import *
from .tools import *
from .yoc import *
from .command import *
from .progress import *
from .subcmds import all_commands
from .cmd import *
from .repo import *



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function
from aostools import *


class Users(Command):
    common = True
    helpSummary = "add or delete user to repo"
    helpUsage = """
%prog [option] <username> [-r <reponame>]
"""
    helpDescription = """
add or delete user to repo
"""

    def _Options(self, p):
        p.add_option('-a', '--add',
                     dest='add', action='store_true',
                     help='add a developer user to repo')
        p.add_option('-d', '--delete',
                     dest='delete', action='store_true',
                     help='delete a developer user to repo')
        p.add_option('-r', '--repo',
                     dest='reponame', action='store', type='string',
                     help='the repo name')

    def Execute(self, opt, args):
        if not (opt.add or opt.delete):
            self.Usage()
            return 1
        username = args[0]
        reponame = None
        yoc = YoC()
        if opt.reponame:
            reponame = opt.reponame
        else:
            yoc.update(True)

        repo = RepoGitee(yoc.conf.gitee_token, yoc.conf.group)
        if opt.add:
            if reponame:
                put_string("Add user:%s to repo:%s." % (username, reponame))
                repo.add_user_to_repo(reponame, username)
            else:
                for component in yoc.occ_components:
                    put_string("Add user:%s to repo:%s." % (username, component.name))
                    repo.add_user_to_repo(component.name, username)
        elif opt.delete:
            if reponame:
                put_string("Delete user:%s to repo:%s." % (username, reponame))
                repo.delete_user_to_repo(reponame, username)
            else:
                for component in yoc.occ_components:
                    put_string("Delete user:%s to repo:%s." % (username, component.name))
                    repo.delete_user_to_repo(component.name, username)
        return 0



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

from aostools import *


class Search(Command):
    common = True
    helpSummary = "Search for components whose name or summary contains <query>"
    helpUsage = """
%prog [option] <query>
"""
    helpDescription = """
Search for components whose name or summary contains <query>
"""

    def _Options(self, p):
        p.add_option('-u', '--update',
                     dest='update', action='store_true',
                     help='update component from git or OCC')

    def Execute(self, opt, args):
        if len(args) != 1:
            self.Usage()
        kword = args[0]
        if is_contain_chinese(kword):
            put_string("Not support search chinese string!")
            exit(-1)

        yoc = YoC()
        if opt.update or yoc.occ_components == None:
            yoc.update(True)

        for component in yoc.occ_components:
            keys = [kword, kword.lower(), kword.upper()]
            search_text = kword.lower()
            if component.name.lower().find(search_text) >= 0:
                component.show(key=keys)
            else:
                component.load_package()
                if component.description.lower().find(search_text) >= 0:
                    component.show(key=keys)
        return 0



# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

from aostools import *
from aostools.tools import get_elf_bin_file
import os
import shutil

class Burn(Command):
    common = True
    helpSummary = "Burn elf or binary file into Flash"
    helpUsage = """
%prog 
"""
    helpDescription = """
Burn elf or binary file into Flash
"""
    def _Options(self, p):
        p.add_option('-b', '--board',
                     dest='board_name', action='store', type='str', default=None,
                     help='get burn tools from selected board')
        p.add_option('-f', '--binfile',
                     dest='bin_file', action='store', type='str', default=None,
                     help='burn specified binary file only')

    def Execute(self, opt, args):
        yoc = YoC()
        change_dir_to_solution(yoc.conf.yoc_path)
        solution = yoc.getSolution(opt.board_name)
        if solution == None:
            put_string("The current directory is not a solution!")
            return 1
        if not solution.flash_program or not os.path.isfile(solution.flash_program):
            put_string("Can not find flash program tool for this solution(or board or chip)!")
            return 1
        if not opt.bin_file:
            data_files = []
            for item in solution.program_data_files:
                one_file = item["filename"] + "#" + hex(item["address"])
                data_files.append(one_file)
            return self.burn_script(solution.flash_program, True, data_files)
        else:
            data_files = [opt.bin_file]
            return self.burn_script(solution.flash_program, False, data_files)
        
    def burn_script(self, script_file, auto_append, data_files):
        cmdstr = '"%s"' % script_file
        if script_file.endswith('.py'):
            cmdstr = 'python "%s"' % script_file
        if auto_append:
            elf_file, bin_file = get_elf_bin_file('SConstruct')
            if elf_file:
                cmdstr += " --elf=\"%s\"" % os.path.abspath(elf_file)
            if bin_file:
                cmdstr += " --bin=\"%s\"" % os.path.abspath(bin_file)
        if data_files:
            for one_file in data_files:
                cmdstr += " --bin=\"%s\"" % one_file
        return os.system(cmdstr) >> 8




# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function

from aostools import *
import os


class Copy(Command):
    common = True
    helpSummary = "Component copy from <old_name> to <new_name>"
    helpUsage = """
%prog <old_name> <new_name>
"""
    helpDescription = """
Component copy from <old_name> to <new_name>.
"""

    def Execute(self, opt, args):
        yoc = YoC()
        if len(args) == 2:
            old_name = args[0]
            new_name = args[1]
            if is_contain_chinese(new_name) or not is_leagal_name(new_name):
                put_string("New name: %s is not valid!" % new_name)
                exit(-1)
            if len(new_name) > 64:
                put_string("New name: %s is too long, max length is 64." % new_name)
                exit(-1)
            component = yoc.components.get(old_name)
            if component:
                if component.copy(new_name):
                    put_string("Component `%s` -> `%s` success." % (old_name, new_name))
                    return 0
                else:
                    put_string("Component copy fail!")
            else:
                put_string("Component `%s` not found!" % old_name)
        else:
            self.Usage()
        return 1




# -*- coding:utf-8 -*-
#
# Copyright (C) 2019-2020 Alibaba Group Holding Limited


from __future__ import print_function
from aostools import *


class Upload(Command):
    common = True
    helpSummary = "Upload components to git or OCC"
    helpUsage = """
%prog [option] [<component>...]
"""
    helpDescription = """
Upload components to git or OCC
"""

    def _Options(self, p):
        p.add_option('-g', '--git',
                     dest='upload_gitee', action='store_true',
                     help='upload code to git repo')
        p.add_option('-o', '--occ',
                     dest='upload_occ', action='store_true',
                     help='upload code to OCC')
        p.add_option('-e', '--aone',
                     dest='upload_aone', action='store', type='string',
                     help='upload code to code.aone')
        p.add_option('-p', '--prefix',
                     dest='prefix', action='store', type='string',
                     help="specify the component name's prefix")
        p.add_option('-t', '--type',
                     dest='type', action='store', type='string',
                     help="specify the component type")
        p.add_option('-m', '--message',
                     dest='message', action='store', type='string',
                     help="commit message")

    def Execute(self, opt, args):
        if not (opt.upload_gitee or opt.upload_occ or opt.upload_aone):
            self.Usage()
            return 1

        yoc = YoC()
        count = len(args)
        if opt.upload_gitee:
            repo = RepoGitee(yoc.conf.gitee_token, yoc.conf.group)
            if repo:
                if count == 0:
                    put_string("Uploading all components, please wait...")
                flag = False
                for component in yoc.components:
                    if component.name in args or count == 0:
                        flag = True
                        component.load_package()
                        ssh_url = repo.create_project(
                            component.name, component.description)
                        if ssh_url:
                            put_string("Uploading %s(%s), please wait..." % (component.name, component.version))
                            component.upload(opt.message)
                            put_string("Upload %s finish." % component.name)
                        else:
                            put_string("Upload %s failed!" % component.name)
                if not flag:
                    put_string("Can't find components:%s!" % str(args))
            else:
                put_string("Connect git repo error!")

        elif opt.upload_occ:
            if count == 0:
                put_string("Uploading all components, please wait...")
                yoc.uploadall()
            else:
                for name in args:
                    put_string("Start to upload component:%s to OCC." % name)
                    yoc.upload(name)
        
        elif opt.upload_aone:
            if count == 0:
                put_string("Uploading all components, please wait...")
            flag = False
            for component in yoc.components:
                if (component.name in args) or count == 0:
                    flag = True
                    component.load_package()
                    if opt.prefix and not component.name.startswith(opt.prefix):
                        continue
                    if opt.type and component.type != opt.type:
                        continue
                    put_string("Uploading %s(%s), please wait..." % (component.name, component.version))
                    component.upload(opt.message, opt.upload_aone)
                    put_string("Upload %s finish." % component.name)
            if not flag:
                put_string("Can't find components:%s!" % str(args))
        return 0
